<?php

/*
Plugin Name: WooCommerce Qazaqpay Gateway
Description: Extends WooCommerce with Qazaqpay payment gateway.
Version: 1.0.0
Author: Qazaqpay
License URI: http://www.gnu.org/licenses/gpl-2.0.html

Text Domain: woocommerce-gateway-qazaqpay
Domain Path: /languages

WC requires at least: 3.2.0
WC tested up to: 8.2.0
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

function init_qazaqpay_gateway_class() {
  class WC_Gateway_Qazaqpay extends WC_Payment_Gateway {
    function __construct()
    {
          $this->id = 'qazaqpay';
          $this->has_fields = false;
          $this->method_title       = 'Qazaqpay';
          $this->method_description = 'Qazaqpay payment gateway solution';

          $this->init_form_fields();
          $this->init_settings();
          
          $this->title              = $this->get_option('title');
          $this->description        = $this->get_option('description');
          $this->enabled            = $this->get_option('enabled');
          $this->agent_id           = intval($this->get_option('agent_id'));
          $this->virtual_pos_id     = intval($this->get_option('virtual_pos_id'));
          $this->payment_type_id    = intval($this->get_option('payment_type_id'));
          
          add_action( 'woocommerce_api_' . $this->id , array( $this, 'webhook' ) );
          add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        }
        
        function webhook() {
          $request_body = file_get_contents( 'php://input' );
          $json = json_decode($request_body, true);

          $order_id = isset($json["billId"]) ? $json["billId"] : null;
          if ( is_null($order_id) ) return;

          $order = wc_get_order( $order_id );
          if ( !$order ) return;

          $order->payment_complete();
          wc_reduce_stock_levels($order_id);
        }

        public function init_form_fields() {
          load_plugin_textdomain('woocommerce-gateway-qazaqpay', false, dirname( plugin_basename( __FILE__ ) ) . '/languages');

          $this->form_fields = array(
              'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce-gateway-qazaqpay'),
                'type' => 'checkbox',
                'label' => __( 'Enable Qazaqpay', 'woocommerce-gateway-qazaqpay' ),
                'default' => 'no'
              ),
              'title' => array(
                'title' => __( 'Title', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'text',
                'description' => __( 'This is the title displayed to the user during checkout', 'woocommerce-gateway-qazaqpay' ),
                'default' => __( 'Qazaqpay', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              ),
              'description' => array(
                'title' => __( 'Description', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'textarea',
                'description' => __( 'This is the description which the user sees during checkout', 'woocommerce-gateway-qazaqpay' ),
                'default' => __("Qazaqpay", 'woocommerce-gateway-qazaqpay'),
                'desc_tip'    => true
              ),
              'gateway_address' => array(
                'title' => __( 'Gateway address', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'text',
                'description' => __( 'Address of gateway', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              ),
              'gateway_login' => array(
                'title' => __( 'Gateway login', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'text',
                'description' => __( 'Login for gateway', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              ),
              'gateway_password' => array(
                'title' => __( 'Gateway password', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'text',
                'description' => __( 'Password for gateway', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              ),
              'agent_id' => array(
                'title' => __( 'Merchant ID', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'number',
                'description' => __( 'Identifier of the Merchant registered in the Payment System IS', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              ),
              'virtual_pos_id' => array(
                'title' => __( 'Virtual cash register ID', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'number',
                'desc_tip'    => true
              ),
              'payment_type_id' => array(
                'title' => __( 'Payment type ID', 'woocommerce-gateway-qazaqpay' ),
                'type' => 'number',
                'description' => __( 'Payment type ID, registered in the Payment System IS', 'woocommerce-gateway-qazaqpay' ),
                'desc_tip'    => true
              )
          );
        }

        function fetch_gateway_token() {
          $http_args = array(
              'headers' => array(
                  'Content-Type' => 'application/json',
              ),
              'body' => json_encode(array(
                  'login' => $this->get_option('gateway_login'),
                  'password' => $this->get_option('gateway_password')
              )),
          );

          $response = wp_remote_post(
            $this->get_option('gateway_address') . "/api/auth/login",
            $http_args
          );

          if( ! is_wp_error( $response ) ) {
              $body = json_decode( $response['body'], true );
              return $body["token"];
          } else {
            return "";
          }
        }

        function init_payment($order, $token) {
          $amount = $order->get_total();

          $body = array(
            "virtual_pos_id" => $this->virtual_pos_id,
            "agent_id" => $this->agent_id,
            "success_url" => $this->get_return_url($order),
            "error_url" => $this->get_return_url($order),
            "notify_url" => $this->get_return_url($order),
            "webhook_url" => get_site_url() . "/?wc-api=qazaqpay",
            "client_info" => array(
                "full_name" => $order->get_formatted_billing_full_name(),
            ),
            "bill" => array(
                "bill_id" => $order->get_id(),
                "sum" => $amount,
                "currency" => "KZT",
                "claims" => array(
                    array(
                      "claim_id" => $order->get_id(),
                      "sum" => $amount,
                      "payment_type_id" => $this->payment_type_id,
                    )
                )
            )
          );
          $http_args = array(
              'headers' => array(
                  'Accept-Language' => 'en',
                  'Content-Type' => 'application/json; charset=utf-8',
                  'Accept' => 'application/json',
                  'Authorization' => 'Bearer ' . $token,
              ),
              'body' => json_encode($body),
          );

          $response = wp_remote_post(
            $this->get_option('gateway_address') . "/api/banks/call_payment_page",
            $http_args
          );

          if( ! is_wp_error( $response ) ) {
            $body = json_decode( $response['body'], true );
            return $body["url"];
          } else {
              return "";
          }
        }

        function process_payment( $order_id ) {
            global $woocommerce;
            $order = new WC_Order( $order_id );
            $status = $order->get_status();
            $token = $this->fetch_gateway_token();
            if ($token == "") {
                wc_add_notice(__( 'An error occured. Try again later', 'woocommerce-gateway-qazaqpay' ), 'notice');
                return;
            }
            $redirect = $this->init_payment($order, $token);
            if ($redirect == "") {
              wc_add_notice(__( 'An error occured. Try again later', 'woocommerce-gateway-qazaqpay' ), 'notice');
              return;
            }
        
            return array(
                'result' => 'success',
                'redirect' => $redirect
            );
        }
    }
}

add_filter( 'woocommerce_payment_gateways', 'register_qazaqpay_gateway_class' );

function register_qazaqpay_gateway_class( $gateways ) {
    $gateways[] = WC_Gateway_Qazaqpay::class;
    return $gateways;
}

add_action( 'plugins_loaded', 'init_qazaqpay_gateway_class' );
